#ifndef LPFILTER_HPP
#define LPFILTER_HPP



/*
 * Copyright (c) 2011 LAAS/CNRS                       --  Oct 2011
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * Description: low-pass, Kalman and other Bayesian filters for perception
 *  
 *  
 *  
 *  Creation date : Jan 19 22:35:29 2011 
 *  Author: HE Wuwei
 *  LAAS-CNRS, Toulouse FRANCE
 */




#include <vector>
#include <deque>

namespace sensFusion_filters
{

class LPfilter {
private:
  int ticInit_;  
  int ticTimeNow_;
  double tInit_; 
  double timePeriod_;
  unsigned historySize_;
  unsigned firstActWindow_;
  std::deque< double > rawHistory_;
  std::deque< double > filHistory_; 
  
public :
  LPfilter();
  LPfilter(const LPfilter&);
  void init(double, double, unsigned, unsigned int);
  void setTime(double&);
  void pushBackSample( double, double);
  void doFiltering();
  void getFilteredResults(double&);
};



class LPfilter3D {
private:
  unsigned int vecSize_; //size for a position 
  double tInit_; //for the filter doesn't slow down the system for the first tics
  double timePeriod_;
  unsigned historySize_;
  unsigned firstActWindow_;
  std::deque< std::vector<double> > rawHistory_;
  std::deque< std::vector<double> > filHistory_;
  int ticInit_;
  int ticTimeNow_;

public :
  LPfilter3D();
  LPfilter3D(const LPfilter3D&);
  void init(double, double, unsigned, unsigned int);
  void setTime(double&);
  void pushBackSample( double, double, double, double);
  void doFiltering(double);
  void getFilteredResults(double&, double&, double&, double&);
};



} //namespace


#endif



